/*
 *  DecoderInterface.h
 *  Copyright (C) 2005 Marcus Overhagen <marcus@overhagen.de>
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef __DECODER_INTERFACE_H_
#define __DECODER_INTERFACE_H_

#include <MediaDefs.h>

typedef status_t (*get_next_chunk_func)(const void **chunkData, size_t *chunkLen, media_header *mh, void *cookie);

class DecoderInterface
{
public:
						DecoderInterface(get_next_chunk_func next_chunk, void *cookie);
	virtual				~DecoderInterface();

	virtual status_t	SetInputFormat(const media_format &in_format) = 0;
	virtual status_t	SetOutputFormat(media_format *output_format) = 0;

	virtual status_t	Decode(void *out_buffer, int64 *out_frameCount,
		          			   media_header *out_mh, media_decode_info *info) = 0;

protected:
	status_t			GetNextChunk(const void **chunkData, size_t *chunkLen, media_header *mh);

private:
	void *				fCookie;
	get_next_chunk_func fGetNextChunk;
};


inline
DecoderInterface::DecoderInterface(get_next_chunk_func next_chunk, void *cookie)
 :	fCookie(cookie)
 ,	fGetNextChunk(next_chunk)
{
}


inline 
DecoderInterface::~DecoderInterface()
{
}


inline status_t
DecoderInterface::GetNextChunk(const void **chunkData, size_t *chunkLen, media_header *mh)
{
	return (*fGetNextChunk)(chunkData, chunkLen, mh, fCookie);
}


extern "C" DecoderInterface * instantiate_mpeg2_video_decoder(get_next_chunk_func next_chunk, void *cookie, const char *param);
extern "C" DecoderInterface * instantiate_mpeg2_audio_decoder(get_next_chunk_func next_chunk, void *cookie, const char *param);
extern "C" DecoderInterface * instantiate_ac3_audio_decoder(get_next_chunk_func next_chunk, void *cookie, const char *param);
extern "C" DecoderInterface * instantiate_deinterlace_filter(get_next_chunk_func next_chunk, void *cookie, const char *param);

#endif
