/*
 *  audio_decoder_addon.h
 *  Copyright (C) 2005 Marcus Overhagen <marcus@overhagen.de>
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included 
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef __AUDIO_DECODER_ADDON_H
#define __AUDIO_DECODER_ADDON_H

#include "../DecoderInterface.h"
#include "../media_header_ex.h"
#include "../libmpg123/mpglib.h"
#include <Locker.h>

class AudioDecoder : public DecoderInterface
{
public:
				AudioDecoder(get_next_chunk_func next_chunk, void *cookie);
				~AudioDecoder();

	status_t	SetInputFormat(const media_format &in_format);
	status_t	SetOutputFormat(media_format *output_format);

	status_t	Decode(void *out_buffer, int64 *out_frameCount,
		               media_header *out_mh, media_decode_info *info);

private:
	status_t	DecodeNextChunk();
	bool		IsValidStream(const uint8 *buffer, int size);
	int			GetFrameLength(const void *header);	

private:
	struct mpstr	fMpgLibPrivate;
	int32			fResidualBytes;
	uint8 *			fResidualBuffer;
	uint8 *			fDecodeBuffer;
	bigtime_t		fStartTime;
	int				fFrameSize;
	int				fFrameRate;
	int				fBitRate;
	int				fChannelCount;
	int				fOutputBufferSize;
	bool			fNeedSync;
	bool			fDecodingError;
};

#endif
